#!/usr/bin/env python

# PiPong - A remake of the classic Pong game using PyGame. Written by
# Liam Fraser - 28/07/2012 for the Linux User & Developer magazine.
# Modified/amended by Joe Barrett (Eurotherm) for the
# 		Brighton Science Festival 2014

import pygame # Provides what we need to make a game
import sys # Gives us the sys.exit function to close our program
import random # Can generate random positions for the pong ball
import wiringpi #to manipulate I/O
import os # To execute the gpio commands from within the program

from pygame.locals import *
from pygame import *

Player1Score = 0
Player2Score = 0

PIN_INPUT_1 = 18
PIN_INPUT_2 = 23
PIN_OUTPUT_1 = 17
PIN_OUTPUT_2 = 22

# Our main game class
class PiPong:
	
	def __init__(self):
		# initialize wiringpi
		os.system("gpio export 17 out")
		os.system("gpio export 22 out")
		os.system("gpio export 18 in")
		os.system("gpio export 23 in")
		os.system("gpio mode 1 in")
		os.system("gpio mode 1 up")
		os.system("gpio mode 4 in")
		os.system("gpio mode 4 up")
		wiringpi.wiringPiSetupSys();
		# Make the display size a member of the class
		self.displaySize = (640, 480)
		
		# Initialize pygame
		pygame.init()
		
		# Create a clock to manage time
		self.clock = pygame.time.Clock()
		
		# Set the window title
		display.set_caption("Pi Pong")
		
		# Create the window
		self.display = display.set_mode(self.displaySize)
			
		# Create the background, passing through the display size
		self.background = Background(self.displaySize)
	
		# Create two bats, three balls and add them to a sprite group
		self.player1Bat = Bat(self.displaySize, "player1")
		self.player2Bat = Bat(self.displaySize, "player2")
		self.ball = Ball(self.displaySize, 0)
		self.ball2 = Ball(self.displaySize, 1)
		self.ball3 = Ball(self.displaySize, 2)
		self.sprites = sprite.Group(self.player1Bat, self.player2Bat,
		self.ball, self.ball2, self.ball3)

		# record previous player scores so that we know when a ball has gone out
		self.PrevPlayer1Score = 0
		self.PrevPlayer2Score = 0
		# counter for input detection
		self.IPCount = 0
		# counters to hold output high
		self.OP1Count = 0
		self.OP2Count = 0
		# set outputs off
		wiringpi.digitalWrite(PIN_OUTPUT_1, 0)
		wiringpi.digitalWrite(PIN_OUTPUT_2, 0)

	def run(self):
		# Runs the game loop
		
		while True:
			# The code here runs when every frame is drawn
					
			# Handle Events
			self.handleEvents()

			# Handle Inputs
			self.handleInputs()

			# Handle Outputs
			self.handleOutputs()
			
			# Draw the background
			self.background.draw(self.display)
						
			# Update and draw the sprites
			self.sprites.update()
			self.sprites.draw(self.display)
			
			# Check for bat collisions
			self.ball.batCollisionTest(self.player1Bat)
			self.ball.batCollisionTest(self.player2Bat)

			# Check for bat collisions (ball2)
			self.ball2.batCollisionTest(self.player1Bat)
			self.ball2.batCollisionTest(self.player2Bat)
			
			# Check for bat collisions (ball3)
			self.ball3.batCollisionTest(self.player1Bat)
			self.ball3.batCollisionTest(self.player2Bat)

			# Update the full display surface to the screen
			display.update()
			
			# Limit the game to 30 frames per second
			self.clock.tick(30)
			
	def handleInputs(self):
		if self.IPCount >= 5:
			input1 = wiringpi.digitalRead(PIN_INPUT_1) # show ball 2 if this input is off (button pressed)
			if input1 == False:
				self.ball2.show()
			input2 = wiringpi.digitalRead(PIN_INPUT_2) # show ball 3 if this input is off (button pressed)
			if input2 == False:
				self.ball3.show()
			self.IPCount = 0
		else:
			self.IPCount = self.IPCount + 1

	def handleOutputs(self):
		if self.OP1Count > 0:
			self.OP1Count = self.OP1Count - 1
			if self.OP1Count == 0:
				wiringpi.digitalWrite(PIN_OUTPUT_1, 0)
		elif Player1Score <> self.PrevPlayer1Score:
			self.OP1Count = 15
			wiringpi.digitalWrite(PIN_OUTPUT_1, 1)
			self.PrevPlayer1Score = Player1Score

		if self.OP2Count > 0:
			self.OP2Count = self.OP2Count - 1
			if self.OP2Count == 0:
				wiringpi.digitalWrite(PIN_OUTPUT_2, 0)
		elif Player2Score <> self.PrevPlayer2Score:
			self.OP2Count = 15
			wiringpi.digitalWrite(PIN_OUTPUT_2, 1)
			self.PrevPlayer2Score = Player2Score
		
	def handleEvents(self):
		
		# Handle events, starting with the quit event
		for event in pygame.event.get():
			if event.type == QUIT:
				pygame.quit()
				sys.exit()
			
			if event.type == KEYDOWN:
				# Find which key was pressed and start moving appropriate bat
				if event.key == K_s:
					# Start moving bat
					self.player1Bat.startMove("down")
				elif event.key == K_w:
					self.player1Bat.startMove("up")
				if event.key == K_DOWN:
					self.player2Bat.startMove("down")
				elif event.key == K_UP:
					self.player2Bat.startMove("up")
				if event.key == K_ESCAPE:
					pygame.quit()
					sys.exit()
				if event.key == K_b: # make ball 2 appear
					self.ball2.show()
				if event.key == K_n: # make ball 3 appear
					self.ball3.show()
				if event.key == K_r: # reset code - including new code to reset the bats
					global Player1Score
					global Player2Score
					Player1Score = 0
					Player2Score = 0
					self.ball.reset()
					self.ball2.reset()
					self.ball3.reset()
					self.player1Bat.reset()
					self.player2Bat.reset()
					self.PrevPlayer1Score = 0
					self.PrevPlayer2Score = 0
					
			
			if event.type == KEYUP:
				if event.key == K_s or event.key == K_w:
					self.player1Bat.stopMove()
				elif event.key == K_DOWN or event.key == K_UP:
					self.player2Bat.stopMove()
				
# The class for the background
class Background:
	
	def __init__(self, displaySize):
		
		# Set our image to a new surface, the size of the screen rectangle
		self.image = Surface(displaySize)
		
		# Fill the image with a green colour (specified as R,G,B) - no, black is better
		self.image.fill((0, 0, 0)) 
		
		# Get width proportionate to display size
		lineWidth = displaySize[0] / 80
		
		# Create a rectangle to make the white line
		lineRect = Rect(0, 0, lineWidth, displaySize[1])
		lineRect.centerx  = displaySize[0] / 2
		draw.rect(self.image, (255, 255, 255), lineRect)

		self.myfont = pygame.font.Font(None,60)
		self.displayx = displaySize[0]
		
	def draw(self, display):
			
		# Draw the background to the display that has been passed in
		display.blit(self.image, (0,0))
		# Scores on top
		scoretext = self.myfont.render(str(Player1Score), 1, (255,255,255))
		display.blit(scoretext, (self.displayx / 4 - 20, 20))
		scoretext = self.myfont.render(str(Player2Score), 1, (255,255,255))
		display.blit(scoretext, (self.displayx * 3 / 4 - 20, 20))


# The class for the bats on either side
class Bat(sprite.Sprite):
	
	def __init__(self, displaySize, player):
			
		# Initialize the sprite base class
		super(Bat, self).__init__()
		
		# Make player a member variable
		self.player = player

		self.displayHeight = displaySize[1]
		
		# Get a width and height values proportionate to the display size
		width = displaySize[0] / 80
		height = displaySize[1] / 6
		
		# Create an image for the sprite using the width and height
		# we just worked out
		self.image = Surface((width, height))
		
		# Fill the image white
		self.image.fill((255, 255, 255))
		
		# Create the sprites rectangle from the image
		self.rect = self.image.get_rect()
		
		# Set the rectangle's location depending on the player
		if player == "player1":
			# Left side
			self.rect.centerx = displaySize[0] / 20
		elif player == "player2":
			# Right side
			self.rect.centerx = displaySize[0] - displaySize[0] / 20
		
		# Center the rectangle vertically
		self.rect.centery = displaySize[1] / 2
		
		# Set a bunch of direction and moving variables
		self.moving = False
		self.direction = "none"
		self.speed = 13
		
	def startMove(self, direction):
		
		# Set the moving flag to true
		self.direction = direction
		self.moving = True
		
	def update(self):
		
		if self.moving:
			# Move the bat up or down if moving
			if self.direction == "up":
				self.rect.centery -= self.speed
				if self.rect.bottom < 20: # stop bat from disappearing off the top
					self.rect.bottom = 20
					self.moving = False
			elif self.direction == "down":
				self.rect.centery += self.speed
				if self.rect.top > self.displayHeight - 20: # stop the bat from disappearing off the bottom
					self.moving = False
					self.rect.top = self.displayHeight - 20
				
	def stopMove(self):

		self.moving = False

	def reset(self): # added to allow a reset of the whole game

		self.rect.centery = self.displayHeight / 2
		

# The class for the ball
class Ball(sprite.Sprite):
	
	def __init__(self, displaySize, ballnumber):
			
		# Initialize the sprite base class
		super(Ball, self).__init__()

		# Get the display size for working out collisions later
		self.displaySize = displaySize
		
		# Get a width and height values proportionate to the display size
		width = displaySize[0] / 30
		height = displaySize[1] / 30
		
		# Create an image for the sprite
		self.image = Surface((width, height))
		
		self.ballnumber = ballnumber
		# Fill the image to colour the ball
		if ballnumber == 2:
			self.image.fill((0, 0, 255)) # blue
		elif ballnumber == 1:
			self.image.fill((255, 0, 0)) # red
		else:
			self.image.fill((255, 255, 255)) # white
		
		# Create the sprites rectangle from the image
		self.rect = self.image.get_rect()
			
		# Work out a speed
		self.speed = displaySize[0] / 110
	
		# Reset the ball
		self.reset()
	
	def reset(self):

		if (self.ballnumber == 0): # the main ball, always showing
			self.showing = 1
			# Start the ball directly in the centre of the screen
			self.rect.centerx = self.displaySize[0] / 2
			self.rect.centery = self.displaySize[1] / 2
		
			# Start the ball moving to the left or right (pick randomly)
			# Vector(x, y)
			if random.randrange(1, 3) == 1:
				# move to left
				self.vector = (-1, 0)
			else:
				# move to right
				self.vector = (1, 0)
		else: # other balls start off hidden
			self.showing = 0
			# Put the ball off the screen
			self.rect.centerx = -100
			self.rect.centery = -100
		
			# Stop the ball moving
			# Vector(x, y)
			self.vector = (0, 0)

	def show(self):
		if (self.showing == 0):
			# Start the ball directly in the centre of the screen
			self.rect.centerx = self.displaySize[0] / 2
			self.rect.centery = self.displaySize[1] / 2
			if self.ballnumber == 2:
				# move to left
				self.vector = (-1, 0)
			else:   # ball number 1
				# move to right
				self.vector = (1, 0)
			self.showing = 1
		
	def update(self):
		if self.showing == 1: # no update if the ball is hidden
			# Check if the ball has hit a wall
			if self.rect.midleft[0] <= 0:
				global Player2Score
				# Hit left side, i.e. right side scores
				self.reset()
				Player2Score = Player2Score + 1
			elif self.rect.midright[0] >= self.displaySize[0]:
				global Player1Score
				# Hit right side, i.e. left side scores
				self.reset()
				Player1Score = Player1Score + 1
			elif self.rect.midtop[1] <= 0:
				# Hit top side
				self.reflectDown()
			elif self.rect.midbottom[1] >= self.displaySize[1]:
				# Hit bottom side
				self.reflectUp()
			
			# Move in the direction of the vector
			self.rect.centerx += (self.vector[0] * self.speed)
			self.rect.centery += (self.vector[1] * self.speed)
	
	def reflectDown(self):
		
		if self.vector[1] < 0: # make sure we don't "bobble" along the top
			deltaX = self.vector[0]
			deltaY = self.vector[1]
			deltaY = -deltaY
			self.vector = (deltaX, deltaY)
			
	def reflectUp(self):
		
		if self.vector[1] > 0: # make sure we don't "bobble" along the bottom
			deltaX = self.vector[0]
			deltaY = self.vector[1]
			deltaY = -deltaY
			self.vector = (deltaX, deltaY)
		
		
	def batCollisionTest(self, bat):
	
		if self.showing == 1:
			# Check if the ball has had a collision with the bat
			if Rect.colliderect(bat.rect, self.rect):
				
				# Work out the difference between the start and end points
				deltaX = self.rect.centerx - bat.rect.centerx
				deltaY = self.rect.centery - bat.rect.centery

				# fix the stuck ball problem
				if deltaX == 0:
					deltaX = self.vector[0]
					

				# Fix the speed in the X direction to 1
				if deltaX < 0:
					deltaX = -1
				else:
					deltaX = 1
				# keep a consistent Y speed, whatever the screen size, maximum of 4
				deltaY = 4.0 * deltaY/(bat.rect.bottom - bat.rect.top)
				
				# Set the ball's new direction
				self.vector = (deltaX, deltaY)
				
			
if __name__ == '__main__':
	game = PiPong()
	game.run()
